# ifndef Frequency_h
# define Frequency_h

# include <Arduino.h>
# include <Adafruit_GFX.h>    // Core graphics library
# include <Adafruit_ST7735.h> // Hardware-specific library
# include <Fonts/FreeSerifBold18pt7b.h>
# include <SPI.h>
# include <RotaryEncoder.h>

enum state { SELECT_MAGNITUDE, CHANGE_DIGIT, LANDSCAPE = 1 };

class Frequency : public RotaryEncoder
{
  state current_state;
  unsigned char digits[8];
  Adafruit_ST7735 &tft;
  unsigned char mag = 0, old_mag = 0;
  int x1, y1;
  unsigned int w, h;
  unsigned long int val = 0;

public:
  Frequency( char but, char p1, char p2, Adafruit_ST7735 &t )
  : RotaryEncoder( but, p1, p2 ), tft( t )
  {
    current_state = SELECT_MAGNITUDE;
    for( unsigned int x = 0; x < sizeof digits; x++ )
      digits[ x ] = 0;
  }

  void
  init( void )
  {
    tft.initR( INITR_GREENTAB );

    tft.fillScreen( ST7735_BLACK );
    tft.setRotation( LANDSCAPE );
    tft.setTextColor( ST7735_WHITE, ST7735_BLACK );
    tft.setFont( &FreeSerifBold18pt7b );
    tft.getTextBounds( (char *)"0", 10, 80, &x1, &y1, &w, &h);
  }

  void
  show_frequency( void )
  {
    tft.setCursor( 10, 80 );
    for( unsigned int x = 0; x < sizeof digits; x++ )
    {
      if( x == mag )
      {
        x1 = tft.getCursorX();
        tft.setTextColor( ST7735_GREEN );
        tft.print( digits[ x ] );
      }
      else
      {
        tft.setTextColor( ST7735_WHITE );
        tft.print( digits[ x ] );
      }
    }
  }

  unsigned long int
  value( void )
  {
    return val;
  }

  void
  value( unsigned long int v )
  {
    val = v;
    int digit = sizeof digits - 1;
    while( digit > 0 )
    {
      digits[ digit-- ] = v % 10;
      v /= 10;
    }
    show_frequency();
  }

  void
  calculate_value( void )
  {
    val = 0;
    for( unsigned int x = 0; x < sizeof digits; x++ )
    {
      val *= 10;
      val += digits[ x ];
    }
  }

  void
  new_digit( void )
  {
    if( position() < 0 )
      position( 0 );
    if( position() > 9 )
      position( 9 );
    digits[ mag ] = position();

    // Erase old character
    int junk;
    tft.getTextBounds( (char *)"0", x1, y1, &junk, &junk, &w, &h );
    tft.fillRect( x1, y1, w + 1, h, ST7735_BLACK );

    tft.setCursor( x1, y1 + h - 1 );
    tft.setTextColor( ST7735_RED );
    tft.print( digits[ mag ] );
    calculate_value();
  }

  void
  new_magnitude( void )
  {
    if( position() < 0 )
      position( 0 );
    if( position() > sizeof digits - 1 )
      position( sizeof digits - 1 );
    mag = position();
    show_frequency();
  }

  virtual
  void
  button_down( void )
  {
    if( current_state == SELECT_MAGNITUDE )
    {
      current_state = CHANGE_DIGIT;
      position( digits[ mag ] );
      new_digit();
    }
    else
    {
      current_state = SELECT_MAGNITUDE;
      position( mag );
      show_frequency();
    }
  }

  virtual
  void
  new_position( void )
  {
    if( current_state == SELECT_MAGNITUDE )
      new_magnitude();
    else if( current_state == CHANGE_DIGIT )
      new_digit();
  }

};

# endif // Frequency_h
