# ifndef Rotary_Encoder_h
# define Rotary_Encoder_h

# include <Arduino.h>

enum { DETENT = 3, LOCKOUT = 2 };

const char
direction_table[] =
{
   0, -1,   1,   0,
   1,  0,   0,  -1,
  -1,  0,   0,   1,
   0,  1,  -1,   0
};

class RotaryEncoder
{
  bool button_state, old_state, changed;
  unsigned char buttonbit, bit1, bit2;
  volatile unsigned char *buttonreg, *reg1, *reg2, last;
  unsigned long old_millis;

  //
  // As the rotary encoder turns, one click causes several changes,
  // adding +1 and -1 sometimes several times. In the end, one click
  // increments by two, so we divide by 2 every time we see the
  // DETENT state.
  //
  long twice_pos, actual_pos;

public:
  RotaryEncoder( char but, char p1, char p2 )
  {
    buttonbit = digitalPinToBitMask( but );
    buttonreg = portInputRegister( digitalPinToPort( but ) );
    bit1 = digitalPinToBitMask( p1 );
    reg1 = portInputRegister( digitalPinToPort( p1 ) );
    bit2 = digitalPinToBitMask( p2 );
    reg2 = portInputRegister( digitalPinToPort( p2 ) );

    pinMode( but, INPUT_PULLUP );
    pinMode( p1, INPUT_PULLUP );
    pinMode( p2, INPUT_PULLUP );

    last = 3;
    twice_pos = 0;
    actual_pos = 0;
    old_millis = millis();
    button_state = false;

    // Enable the pin interrupt, clear any interrupts, enable interrupts
    *digitalPinToPCMSK( but ) |= bit( digitalPinToPCMSKbit( but ) );
    PCIFR  |= bit (digitalPinToPCICRbit( but ));
    PCICR  |= bit (digitalPinToPCICRbit( but ));

    *digitalPinToPCMSK( p1 ) |= bit( digitalPinToPCMSKbit( p1 ) );
    PCIFR  |= bit (digitalPinToPCICRbit( p1 ));
    PCICR  |= bit (digitalPinToPCICRbit( p1 ));

    *digitalPinToPCMSK( p2 ) |= bit( digitalPinToPCMSKbit( p2 ) );
    PCIFR  |= bit (digitalPinToPCICRbit( p2 ));
    PCICR  |= bit (digitalPinToPCICRbit( p2 ));
    changed = true;
  }

  long position()
  {
    return actual_pos;
  }

  void position( long p )
  {
    actual_pos = p;
    twice_pos = p * 2;
  }

  bool
  button( void )
  {
    return button_state;
  }

  virtual
  void
  button_down( void )
  {
  }

  virtual
  void
  new_position( void )
  {
  }

  void pushed( void )
  {
    changed = true;
    bool test = (*buttonreg & buttonbit) ? false : true;
    //
    // Read the bit, and then ignore any further transitions for LOCKOUT
    // milliseconds. Thus we ignore any bounces the switch makes.
    //
    if( millis() - old_millis > LOCKOUT )
    {
      button_state = test;
      old_millis = millis();
      if( button_state )
      {
        button_down();
      }
    }
  }

  void rotated( void )
  {
    changed = true;
    int change1 = (*reg1 & bit1) ? 1 : 0;
    int change2 = (*reg2 & bit2) ? 2 : 0;
    char current = change2 + change1;

    if( current != last )
    {
      twice_pos += direction_table[ last * 4 + current ];
      last = current;

      if( last == DETENT )
      {
        actual_pos = twice_pos / 2;
        new_position();
      }
    }
  }
};

# endif // Rotary_Encoder_h
