# include <RotaryEncoder.h>

struct ShowIt : public RotaryEncoder
{
  int button_press_count = 0;

  ShowIt( char button_push_pin, char rotation_pin_1, char rotation_pin_2 )
    : RotaryEncoder( button_push_pin, rotation_pin_1, rotation_pin_2 )
  {
  }

  //
  // This is called whenever the button is pushed down
  //
  void
  button_down( void )
  {
    button_press_count++;
  }
  
};

//
// We use pin 3 for the button, and pins A2 and A3 for rotation
//
ShowIt my_encoder( 3, A2, A3 );

//
// Set up the interrupt routines for rotation and button presses
//
ISR( PCINT1_vect )
{
  my_encoder.rotated();
}

ISR( PCINT2_vect )
{
  my_encoder.pushed();
}



void
setup( void )
{
  Serial.begin( 115200 );
  Serial.println( "Rotate the encoder and push the knob down a few times" );
}

long int old_val = 0;
int old_count = 0;

void
loop( void )
{
  if( my_encoder.position() != old_val )
  {
    Serial.print( "Value is " );
    Serial.println( my_encoder.position() );

    old_val = my_encoder.position();
  }

  if( my_encoder.button_press_count != old_count )
  {
    Serial.print( my_encoder.button_press_count );
    Serial.println( " button pushes" );

    old_count = my_encoder.button_press_count;
  }
}
